//_______________________________________________________________________________________________

//COUNT ON-IT! The premier ImageJ tool for counting colonies arrayed in a rectangular grid. 
//Written by Ren Dodge, Ludington Lab, Carnegie Institute of Washington, 2019-2021
//Built for high throughput counting of 96 spot rectangular plates, but capable handling any number of uniform grid of blobs.

//macro "Count Gridded Plates" is the principal macro. This utilizes all the features of count on it...
//First a file must be open
//User may choose to open a whole folder to process using a representative image, start from the current image, or process just one image.

//user may batch process, or "assisted batch" counting to allow for error checking. 
//Initialize parameters and make selections as with the circular plate counter
//Apply the grid overlay after the colony overlay
//Option to batch process plates



macro "COUNT ON-IT Grid Plate" {

sw=screenWidth;
sh=screenHeight;

//INITIALIZE IJ PREFERENCES
run("Colors...", "selection=yellow");
																	//LOG start
																	print("0. Count On-It Started"); //LOG the process
																	getDateAndTime(year, month, dayOfWeek, dayOfMonth, hour, minute, second, msec);
																	print("y"+year+".m"+month+".d"+dayOfMonth+".h"+hour+".m"+minute+".s"+second);

//INITIALIZE WHILE LOOP --- Initialize this loop, which allows a redo of all initialization settings, if chosen, or to continue on to process more. 
nextMove="Redo";
while(nextMove=="Redo") {
		setBatchMode(false);

		// 1. Initialization sets parameters for the whole batch  __________________________________________________________________________________________________________1	
		
		//Chose what channels, border, grid, etc, to use, other parameters
							
							Dialog.create("COUNT ON-IT");
							Dialog.setLocation((sw*0.25),(sh*0.25));
		
							Dialog.addMessage("    >°))))彡\n   ******************  COUNT ON-IT  *****************");
							   
							Dialog.addMessage("   The high-throughput colony counting suite for Fiji\n  *********************************************************");

							//Dialog.addSlider("<html><b>Colony Threshold: </b>", 0.5, 1.0, 0.85);//Slider
							
							Dialog.addChoice("Channel:", newArray("grays", "red", "green", "blue"));//Channel
							
							Dialog.addNumber("Min Size:", 50);
							Dialog.addString("Max Size:", "Infinity");
							Dialog.addNumber("Max Aspect:", 2.5);
							
							//Dialog.addChoice("Selection Method:", newArray("watershed", "puddle"));
							Dialog.addString("Suffix", "-COI-Receipt", 24)
							Dialog.addString("Overlay Text", "COUNT ON-IT 2.0 RD", 24)
							
							Dialog.addMessage("\nGrid Settings: "); //Grid
							Dialog.addNumber("Number of Rows:", 8);
							Dialog.addNumber("Number of Columns:", 12);
							Dialog.addNumber("Gap Size:", 10);
							Dialog.addNumber("Border Size:", 30);
							
							items=newArray("Batch entire folder", "Start from current image", "Process single image" ); //Batch Mode
 							Dialog.addChoice("Source:", items,"Batch Process a Folder");

							Dialog.addMessage("*Crop images with CROPTACTULAR before processing \n    ****************************************************\nPress OK to continue");
							Dialog.show();
											minMeanCircularity = 0.85;
											channel = Dialog.getChoice();
											mincolony = Dialog.getNumber();
											maxcolony = Dialog.getString();
											maxAspect = Dialog.getNumber();
											
											//selectionMethod = Dialog.getChoice();
											suffix=Dialog.getString();
											Otext=Dialog.getString();
											//For the grid-maker
										 	rows = Dialog.getNumber();
											cols = Dialog.getNumber();
											gap = Dialog.getNumber();
											border = Dialog.getNumber();
											startpoint= Dialog.getChoice();//whether to process whole folder, remaining images, or single image. 
												
												
		// 2. FILE HANDLING ______________________________________________________________________________________________________________________________________2						
		
		//If a file is open that is considered the representative image, otherwise an open new file window will launch.
		if(isOpen(File.name)){
			title=getTitle;
		}else{
		open(); //open a file
		}
		//get a list of files to be processed
		title=getTitle(); //get the current filename
		dir=getDirectory("image"); //get current directory
		templist = getFileList(dir); //get files in the directory
		fileList=validFileXList(templist); //make a new list of only valid files

			//if (startpoint=="Batch process a folder"){ }
			if (startpoint=="Start from current image"){//if starting from current image 
				I=ArrayIndex(fileList, title);
				fileList=Array.slice(fileList,I,fileList.length);//we have sliced off all files preceeeding current one from the list
				}
				
		numFiles=lengthOf(fileList);//number of files to process
		
		selectWindow(title); //make it look nice...
		run("Maximize");
		run("Scale to Fit");
																					//LOG fileList
																					print("2. Files:\n");for (i = 0; i < numFiles; i++) {print(fileList[i]);} 
																							
		// 3. FIRST IMAGE MANUAL SETTINGS  size, crop threshold..._________________________________________________________________________________________________3
		
		/* Option for selecting colony sie with oval tool 
		 *  
		mincolony = getArea("min"); //Get that minimum area
		maxcolony = getArea("max");
		 */
	
		run("Duplicate...", "title=Overlay_"+title); //make duplicate file for use as an overlay
		overlayTitle=getTitle();
		selectWindow(title);

		//INSERT CROP FEATURE and Selection Bounds

		if (channel=="grays"){ // Split Channels make the image 8-bit, either grayscale or split and choose one channel. 
			run("8-bit");
			}else{
				run("Split Channels");
				selectWindow(title+" ("+channel+")");
				title=getTitle();
					};	
		
		if(roiManager("count")>0){roiManager("reset");}	//Clear any old junk before thresholding	
		run("Clear Results");
		run("Remove Overlay");	
					
		
		run("Threshold...");
		setAutoThreshold("Default dark");
		selectWindow("Threshold");
		waitForUser("Set Threshold");
			
		getThreshold(lowcut,highcut);
		setThreshold(lowcut,highcut);

																		print("3.Manual Settings:\n mincolony:"+mincolony+" maxcolony:"+maxcolony+" lowcut:"+lowcut+" highcut:"+highcut);
					
		// 4. ANALYZE PARTICLES - GET RESULTS select colonies using watershed _______________________________________________________________________4


		//WATERSHED + SELECT + ANALYSE
		watershedSelector();
		

					
//***** MACRO level results into some arrays *****

Area=results2array("Area");
Circ=results2array("Circ.");
AR=results2array("AR");
XM=results2array("XM");
YM=results2array("YM");

n=0; //numberof files processed so far
gridpx=((getWidth()-gap)*(getHeight()-gap))/(rows*cols); //number of pixels in a grid.
print("gridpx"+gridpx);

plA=plateMeanAcirc(); // Array of circular ROIs - Call this function to get the stats for the whole plate (also could get stats for raw areas)
Array.getStatistics(plA, plmin, plmax, plM, plstdDev);//get some macro level stats

																if(plM<=0){print("no Colonies Detected");}

																print("4.Analyzed particles");
																
		// 5. PROCESS RESULTS - Quantify colonies, make a table of those results, produce an overlay_________________________________________________________________________________5
		C=newArray(nResults);
						f="";//this is the suffix appended to the filename in the column headers - for the first image it is nothing
		
//...GENERATE A NEW COUNT ON-IT RESULTS TABLE...Added to by the makegridCount function.

sourceDir=File.getName(File.directory);

						table1 = "COUNT ON-IT Summary "+sourceDir;
						Table.create(table1);
						A1pos=letnumCoordArray();
						Table.setColumn("Position", A1pos );			
						Table.update;
					
		run("Add Image...", "image="+overlayTitle+" x=0 y=0 opacity=100");

		makegridCount(); // VERY IMPORTANT function Draws The Grid AND counts the colonies in each grid (its kind of a beast!!!)
		
		array2results("Colonies/ROI", 0, C);
	
		changeoverlay(C); // Add the overlay to the image

		dataWriteOn(); // append a footer to the iamge with some useful information. 
									print("5. Colonies counted total#="+sum(C));
		// 6. INSPECT RESULTS AND CHOOSE NEXT MOVE ____________________________________________________________________________________________________________________________________6
		selectWindow(title);
		waitForUser("Inspect Results"); // Inspecting the image + overlay...

							Dialog.create("COUNT ON-IT 1.1");
							Dialog.addMessage("Next, select a folder to save receipt images \nand the Colony Counter Results table.");
							items=newArray("Proceed", "Redo", "Save and Exit");
							Dialog.addChoice("\n      Do you want continue or redo the parameters setup?", items);
							Dialog.show();
							
							nextMove=Dialog.getChoice();
		
				if(nextMove=="Redo"){
					selectWindow(overlayTitle);
					close();
					selectWindow(table1);
					run("Close");
					selectWindow(title);
					run("Revert");
					run("Remove Overlay");
					run("Select None");
				}
				
				if((nextMove=="Save and Exit")||(startpoint=="Process single image")){
					run("Jpeg...");
					selectWindow(table1);
					saveAs();
					exit();
				}
				

} //...End WHILE LOOP...
		newTitle=File.nameWithoutExtension;
		saveDir=getDirectory("Choose a Directory to Save Results");			
		saveAs("Jpeg", saveDir+newTitle+suffix);
		run("Close All");


// 7. BATCH LOOP FOR REMAINING IMAGES __________________________________________________________________________________________________________________________________________________ 7
				
fileList=Array.deleteValue(fileList, title);//remove the first image from the list
								
	for (file=0; file < fileList.length; file++) {

						
						f=file; //the file number for column headers
						
						open(dir+fileList[file]);//open the first file
						
						title=getTitle; //get the new title
						
						run("Duplicate...", "title=Overlay_"+title);
						overlayTitle=getTitle();
						selectWindow(title);

						/*JUNK
						 * 
						 mincolony = getArea("min"); //Get that minimum area
						maxcolony = getArea("max");
		*/
		
						//select the correct channel to use
						if (channel=="grays"){
							run("8-bit");
							}else{
								run("Split Channels");
								selectWindow(title+" ("+channel+")");
								title=getTitle;
									};

		//INSERT CROP HERE
						
							//clear the results table, make a new selection
							run("Clear Results");
							
							if(roiManager("count")>0){
								roiManager("reset");
								}
			
								run("Remove Overlay");
						
						setThreshold(lowcut,highcut);
						
							selectWindow("Threshold");
							waitForUser("Set Threshold");
							getThreshold(lowcut,highcut);
							
						setThreshold(lowcut,highcut);
						
						//Analyze Particles with the selected method
						
		watershedSelector();
		

				//Set some array variables from the Results table
Area=results2array("Area");
Circ=results2array("Circ.");
AR=results2array("AR");
XM=results2array("XM");
YM=results2array("YM");


plA=plateMeanAcirc(); // Call this function to find make an array of circular ROIs, then get stats
Array.getStatistics(plA, plmin, plmax, plM, plstdDev);//get some macro level stats

																if(plM<=0){print("no Colonies Detected");}

		C=newArray(nResults);
				
		run("Add Image...", "image="+overlayTitle+" x=0 y=0 opacity=100");
								
		makegridCount(); // VERY IMPORTANT function Draws The Grid AND counts the colonies in each grid (its kind of a beast!!!)
	
		array2results("Colonies/ROI", 0, C);
	
		changeoverlay(C); // Add the overlay to the image
	
		dataWriteOn(); // append a footer to the iamge with some useful information. 
		
		selectWindow(title);
		waitForUser("OK or ESC");
								
		newTitle=File.nameWithoutExtension;
		saveAs("Jpeg", saveDir+newTitle+suffix);
		n++;																	print("Saved as:"+saveDir+newTitle+suffix);
		run("Close All");					
								
				} //end batch loop------------------------------
				
				
// 8. Save table of reults

Table.save(saveDir+table1+".csv");	
				
//  MACRO END
}






//---------------FUNCTIONS---------------------------------------


//SELECTION ALGORITHMS 

//Erode selekction aka. Puddling makes a selction of minority regions by eroding extraneous pixels, then amplifying the remaing pixels to recover blobs.  
function watershedSelector(){

		blur=round(getWidth()/(1500*minMeanCircularity));
		run("Remove Overlay");	
		setOption("BlackBackground", false);	
		run("Convert to Mask");
		run("Gaussian Blur...", "sigma=blur");
		run("Make Binary");
		run("Watershed");
		run("Set Measurements...", "area center shape display redirect=None decimal=2");
		run("Analyze Particles...", "size=mincolony-Infinity circularity=mincircle-1.00 show=Overlay display add");
		}


//COLONY COMPUTER FUNCTIONS

//MAKEGRID!!! GRID AND COUNT makes a grid and counts it at the same time! probably will overlay 

function makegridCount(){

	run("Overlay Options...", "stroke=cyan width=3 fill=none");
	setColor("cyan");
	setFont("SansSerif", 60, "bold");
		
	run("Select All"); // maybe this could be of a select region for some reason...
	getSelectionCoordinates(a, b);
	
	
	
	// 1.Initialize wariables for grid ... 
	w=(a[1]-a[0])/cols; //width of a cell
	h=(b[2]-b[1])/rows; //height of a cell
	
	x=newArray(cols); 
	for(i=0; i<(cols);i++){x[i]=a[0]+(w*i);}//find x coordinates sans border or gap
	
	y=newArray(rows); 
	for(j=0; j<(rows); j++){y[j]=b[0]+(h*j);} //find y coordinates without border or gap
	
	
	WholeAreas=newArray(rows*cols); // Array the total area of all regions in a cell
	RawCount=newArray(rows*cols); // All the regions in a cell, before filtering
	r=0; //initialize the row index for these arrays rows(rowson the output table) this is going to equal grid rows*cols.
	//...
	
	//2. Get the total areas and the total count of all regions in each cell...
		
		makeRectangle(x[0]+border, y[0]+border, w-border-gap/2, h-border-gap/2); // A1. draw the upper left
		WholeAreas[r]=AreaXY(); 
		RawCount[r]=rawCount();
		r++;
	
		// A. draw the first row
		for(i=1; i<(cols-1); i++){
			makeRectangle(x[i]+gap/2, y[0]+border, w-gap, h-border-gap/2);
			WholeAreas[r]=AreaXY();
			RawCount[r]=rawCount();
			r++;	
			}
			// A12. draw the upper right
			makeRectangle(x[cols-1]+(gap/2), y[0]+border, w-border-gap/2, h-border-gap/2);
			WholeAreas[r]=AreaXY();
			RawCount[r]=rawCount();
			r++;
		
			//B1-B12...G12
	
		//B1...
		for(j=1; j<(rows-1);j++){
				//first
				makeRectangle(x[0]+border, y[j]+gap/2, w-border-gap/2, h-gap);
				WholeAreas[r]=AreaXY();
				RawCount[r]=rawCount();
				r++;
				// B2-11... middle
				for(i=1; i<(cols-1);i++){
					makeRectangle(x[i]+gap/2, y[j]+gap/2, w-gap, h-gap);
					WholeAreas[r]=AreaXY();
					RawCount[r]=rawCount();
					r++;	
					}
				//last B12...
				makeRectangle(x[cols-1]+gap/2, y[j]+gap/2, w-border-gap/2, h-gap);
				WholeAreas[r]=AreaXY();
				RawCount[r]=rawCount();
				r++;
				}
	
			//H1.draw the lower left
			makeRectangle(x[0]+border, y[rows-1]+gap/2, w-border-gap/2, h-border-gap/2);
			WholeAreas[r]=AreaXY();
			RawCount[r]=rawCount();
			r++;	
				
		//H2-11.draw the last row
		for(i=1; i<(cols-1);i++){
				makeRectangle(x[i]+gap/2, y[rows-1]+gap/2, w-gap, h-border-gap/2);
				WholeAreas[r]=AreaXY();
				RawCount[r]=rawCount();
				r++;
				}
			//H12.draw the lower right
			makeRectangle(x[cols-1]+gap/2, y[rows-1]+gap/2, w-border-gap/2, h-border-gap/2);
			WholeAreas[r]=AreaXY();
			RawCount[r]=rawCount();
			r++;

	//...
	//PART 2 OF MAKEGRIDCOUNT... 1-2-3-4-5-6-7-8-9-10-11...12!
	
	spotCounts=newArray(rows*cols);
	minC=newArray(rows*cols);
	maxC=newArray(rows*cols);
	M=newArray(rows*cols);
 	stdDev=newArray(rows*cols);	
	r=0;
		
	// A1. draw the upper left
		makeRectangle(x[0]+border, y[0]+border, w-border-gap/2, h-border-gap/2);
		run("Add Selection...");
		R=countColoniesXYselection(C, WholeAreas[r], RawCount[r]); // This will do some calculations on the arrays we just made. returns array R; 0] spotCounts, [1] circCOunts, [2] AreasCircs.
		spotCounts[r]=R[0];minC[r]=R[1]; maxC[r]=R[2]; M[r]=R[3]; stdDev[r]=R[4];
		Overlay.drawString(spotCounts[r], x[0]+border, y[0]+border+68);
		r++;
	
	// A. draw the first row
		for(i=1; i<(cols-1); i++){
		makeRectangle(x[i]+gap/2, y[0]+border, w-gap, h-border-gap/2);
		run("Add Selection...");
		R=countColoniesXYselection(C, WholeAreas[r], RawCount[r]);
		spotCounts[r]=R[0];minC[r]=R[1]; maxC[r]=R[2]; M[r]=R[3]; stdDev[r]=R[4];
		Overlay.drawString(spotCounts[r], x[i]+gap/2, y[0]+border+68);
		r++;	
		}
	// A12. draw the upper right
		makeRectangle(x[cols-1]+(gap/2), y[0]+border, w-border-gap/2, h-border-gap/2);
		run("Add Selection...");
		R=countColoniesXYselection(C, WholeAreas[r], RawCount[r]);
		spotCounts[r]=R[0];minC[r]=R[1]; maxC[r]=R[2]; M[r]=R[3]; stdDev[r]=R[4];
		Overlay.drawString(spotCounts[r], x[cols-1]+(gap/2), y[0]+border+68);
		r++;
	
		//B1-B12...G12
	
		//B1...
		for(j=1; j<(rows-1);j++){
			//first
			makeRectangle(x[0]+border, y[j]+gap/2, w-border-gap/2, h-gap);
			run("Add Selection...");
			R=countColoniesXYselection(C, WholeAreas[r], RawCount[r]);
			spotCounts[r]=R[0];minC[r]=R[1]; maxC[r]=R[2]; M[r]=R[3]; stdDev[r]=R[4];
			Overlay.drawString(spotCounts[r], x[0]+border, y[j]+(gap/2)+68);
			r++;
			// B2-11... middle
			for(i=1; i<(cols-1);i++){
					makeRectangle(x[i]+gap/2, y[j]+gap/2, w-gap, h-gap);
					run("Add Selection...");
					R=countColoniesXYselection(C, WholeAreas[r], RawCount[r]);
					spotCounts[r]=R[0];minC[r]=R[1]; maxC[r]=R[2]; M[r]=R[3]; stdDev[r]=R[4];
					Overlay.drawString(spotCounts[r], x[i]+gap/2, y[j]+(gap/2)+68);
					r++;	
					}
			//last B12...
			makeRectangle(x[cols-1]+gap/2, y[j]+gap/2, w-border-gap/2, h-gap);
			run("Add Selection...");
				R=countColoniesXYselection(C, WholeAreas[r], RawCount[r]);
				spotCounts[r]=R[0];minC[r]=R[1]; maxC[r]=R[2]; M[r]=R[3]; stdDev[r]=R[4];
			Overlay.drawString(spotCounts[r], x[cols-1]+gap/2, y[j]+(gap/2)+68);
			r++;
			}
	
		
	//H1.draw the lower left
		makeRectangle(x[0]+border, y[rows-1]+gap/2, w-border-gap/2, h-border-gap/2);
		run("Add Selection...");
					R=countColoniesXYselection(C, WholeAreas[r], RawCount[r]);
					spotCounts[r]=R[0];minC[r]=R[1]; maxC[r]=R[2]; M[r]=R[3]; stdDev[r]=R[4];
					Overlay.drawString(spotCounts[r], x[0]+border, y[rows-1]+(gap/2)+68);
					r++;	
	//H2-11.draw the last row
		for(i=1; i<(cols-1);i++){
					makeRectangle(x[i]+gap/2, y[rows-1]+gap/2, w-gap, h-border-gap/2);
					run("Add Selection...");	
					R=countColoniesXYselection(C, WholeAreas[r], RawCount[r]);
					spotCounts[r]=R[0];minC[r]=R[1]; maxC[r]=R[2]; M[r]=R[3]; stdDev[r]=R[4];
					Overlay.drawString(spotCounts[r], x[i]+gap/2, y[rows-1]+(gap/2)+68);
					r++;
		}
	//H12.draw the lower right
					makeRectangle(x[cols-1]+gap/2, y[rows-1]+gap/2, w-border-gap/2, h-border-gap/2);
					run("Add Selection...");
					R=countColoniesXYselection(C, WholeAreas[r], RawCount[r]);
					spotCounts[r]=R[0];minC[r]=R[1]; maxC[r]=R[2]; M[r]=R[3]; stdDev[r]=R[4];
					Overlay.drawString(spotCounts[r], x[cols-1]+gap/2, y[rows-1]+(gap/2)+68);
					r++;
					run("Select None");
					
											
											
								// ...TABLE UPDATE ... put those spotcounts in the table
								label=File.nameWithoutExtension;
								shortColhead=substring(label, 0, 8)+"-";
								Table.setColumn(label+" CFUs", spotCounts);

								/*
								if(nRf){Table.setColumn(shortColhead+" nResults", RawCount);}
								if(Mf){Table.setColumn(shortColhead+" Mean", M);}
								if(minf){Table.setColumn(shortColhead+" MIN", minC);}
								if(maxf){Table.setColumn(shortColhead+" MAX", maxC);}
								if(SDf){Table.setColumn(shortColhead+" StdDev", stdDev);}
								if(TAf){Table.setColumn(shortColhead+" TotArea", WholeAreas);}
								*/
								
								Table.update;

						
													
	rectBounds=newArray(a[0], b[0], (a[1]-a[0]),(b[2]-b[1]));

	return rectBounds;
	}

//FUNCTIONS FOR MAKEGRIDCOUNT___________________________________________________________MGC FUNCTIONS

function AreaXY(){
	areasum=0;
	for (i=0;i<nResults; i++){ 
		if(selectionContains(XM[i], YM[i])){
			areasum=areasum+Area[i];
		}}
	return areasum;
	}

function rawCount(){
	rawc=0;
		for (i=0;i<nResults; i++){ 
			if(selectionContains(XM[i], YM[i])){
				rawc++;
			}}
			return rawc;
		}

function countColoniesXYselection(C, WholeArea, RawCount){	//Adds colonies to the array C, which contains all the Colonies/ROI on the plate.
	ColonyCountXY=0;
	CircAreasXY=minCircsArrayinXY(); // get the array of areas with > tolerance circulatrity
	Array.getStatistics(CircAreasXY, minC, maxC, M, stdDev);
		// * M is the mean size of a circle within that selection - it is used to normalize the size of blobs(ROIs) to colonies/ROI
		// * minC is the smallest circle blob in that selection
		// * maxC is the smallest circle blob in that selection
		// * stdDev is the standard deviation. 
		// * WholeAreas and RawCount are evaluated within that XY selection along with stats to determine ColonyCountXY, arguments to function.

		//10% Rejection Rules
if((CircAreasXY.length==2)&&(WholeArea>(0.1*gridpx))){ColonyCountXY="N/A";} else{ // If there is one colony it must be less than 10% of a cell area.
		if(sum(CircAreasXY)==0&&WholeArea<plM){ColonyCountXY=0;} else{
		if(sum(CircAreasXY)<(0.1*WholeArea)){ColonyCountXY="N/A";}// The sum of circular areas must be at least 10% of the total colony area in the cell.
				 else{ //unless it is just a little bit of shit, then it's just zero.
					if(M>(10*plM)){ColonyCountXY="E3";}  // The mean circle size may not be greater than 10X the plate-wide mean.
	
	else{ 

	
	for (i=0;i<nResults;i++){             //evaluate all results, but just alter the ones within the selection.
	if(selectionContains(XM[i], YM[i])){
			
			if(AR[i]<maxAspect){
				if(Circ[i]>minMeanCircularity){C[i]=1;}else{
				C[i]=-floor(-(abs(Area[i]-stdDev)/(M*minMeanCircularity)));	
				if(C[i]==0) {C[i]=1;}
				}
				ColonyCountXY=ColonyCountXY+C[i];
			}

			if(WholeArea==0){C[i]=0; ColonyCountXY=0;}//if there's nothing, there's nothing...
			
		}}}}}}
		return newArray(ColonyCountXY, minC, maxC, M, stdDev);
}


// Circ is the macro level array containing the results column "Circ." Area is: Results "Area" minMeanCircularity is the macro level variable.

function minCircsArrayinXY() { //Returns an array of areas for circles greater than the minMeanCircularity threshold (0-1).

A=newArray(1);
A[0]=0;

	for (i=0;i<nResults; i++){ 
	if(selectionContains(XM[i], YM[i])&&Circ[i]>minMeanCircularity){
		if(i==0){A[0]=Area[i];} else{
		  A = Array.concat(A, Area[i]);}
		}}
		return A; //Returns an array of areas for circles greater than the minMeanCircularity threshold (0-1).
}


function plateMeanAcirc(){
plA=newArray(1);
plA[0]=0;

	for (i = 0; i < nResults; i++) {
	if(Circ[i]>minMeanCircularity){
		  plA = Array.concat(plA, Area[i]);
		}}
		return plA; //Returns an array of areas for circles greater than the minMeanCircularity
}

//change overlay changes the overlay of an image, input is an array, assumes all the results are present in the ROI manager. 
//Renames the labels and changes overlay colors based on the results. Function is applied to last regions number equal to size of array

function changeoverlay(anArray){
	
	roiManager("UseNames", "true");
	roiManager("Set Line Width", 2);
	
	for (i=0;i<roiManager("count");i++){
		roiManager("Select", i);
		roiManager("Rename", "");
		roiManager("Set Color", "white");
		}
	
	for (i=0;i<roiManager("count");i++){
		roiManager("Select", i);
	
	if(anArray[i]==0){roiManager("Rename", "-");
		roiManager("Set Color", "cyan");
		;}
	
	if(anArray[i]==1){roiManager("Rename", i+1);
		roiManager("Set Color", "yellow");
		}
	
	if(anArray[i]>1){roiManager("Rename", i+1+"(+"+anArray[i]+")");
		roiManager("Set Color", "red");
		}
		
	}
	
	}

//Make an array with length colXrows containg a sequence of letters equal to rows with integers up to number of columns

function letnumCoordArray(){
	Letters=newArray("A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","W","R","S","T","U","V","W","X","Y","Z");
	Numbers=newArray("1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20","21","22","23","24","25","26");
	
	Coords=newArray(rows*cols);
	
	for(i = 0; i < Coords.length; 0) {
				for (h = 0; h < rows; h++){
						for (j = 0; j < cols; j++) {
							Coords[i]=Letters[h]+Numbers[j];
							i++;
						}}}
				return Coords;
	}

function results2array(ColumnTitle){
	A = newArray(nResults);
	for(i=0; i<nResults; i++){
		A[i] = getResult(ColumnTitle, i);	
	}
	return A;
	}


//MISC. FUCTIONS

//accepts a File List as input, returns a list of files 
function validFileXList(templist){
		listlength=0;
	for(i=0; i<templist.length; i++){
		if (endsWith(templist[i], ".JPG")||endsWith(templist[i], ".jpg")||endsWith(templist[i], ".tif")||endsWith(templist[i], ".tiff")||endsWith(templist[i], ".png")||endsWith(templist[i], ".gif")) {listlength++;}}

	//put those files in the new array, only these will be processed
			list=newArray(listlength);
			l=0;
			
	for(i=0; i<templist.length; i++){
		if  (endsWith(templist[i], ".JPG")||endsWith(templist[i], ".jpg")||endsWith(templist[i], ".tif")||endsWith(templist[i], ".tiff")||endsWith(templist[i], ".png")||endsWith(templist[i], ".gif")) {
			list[l]=templist[i];
			l++;}}
	return list;
	}

//ARRAY FUNCTIONS

//given a term, this function will return the first index which contains that term
function ArrayIndex(AR, term){
	for (i = 0; i < AR.length; i++) {
		if(AR[i]==term){return i;}
	}
	return 0;
}

//puts a result column into an array - argument a column name, it returns the array
function results2array(ColumnTitle){
	A = newArray(nResults);
	for(i=0; i<nResults; i++){
		A[i] = getResult(ColumnTitle, i);	
	}
	return A;
	}

//puts an array in the Results table - arguments are the column name, the row to start with (usually 0) and the array. Returns nothing but Results.

function array2results (column, firstrow, anArray){
	selectWindow("Results");
	for (i=0;i<lengthOf(anArray);i++){ 
		setResult(column, firstrow+i ,anArray[i]);
		//print(anArray[i]);
		
	}
	updateResults();
}

//returns the SUM of an array
function sum(anArray){
	total=0;
	for (i=0;i<lengthOf(anArray);i++){ 
	total=total+anArray[i];
	}
	return total; 
	}

//find the MEAN of an array
function mean (anArray){
	average=sum(anArray)/lengthOf(anArray);
	return average;
	}

//Prompts user to make an ellipse selction (actually could use any selection) Returns the area of that selection in pixels. Adds to reults table, then deletes. 
function getArea(minmax){
	setTool("oval");
	waitForUser("COUNT ON-IT", "Select the "+minmax+" colony size with ellipse tool");
	getSelectionBounds(xx, yy, ww, hh);
	area=PI*(ww*hh*0.25);
	run("Select None");
	
	return area; 
}

function dataWriteOn(){
	setBackgroundColor(0, 0, 0);
	w=getWidth();
	h=getHeight();
	hN=h+100;
	run("Set Scale...", "distance=w known=110 unit=mm");
	run("Canvas Size...", "width=w height=hN position=Top-Center");
	
	
	getDateAndTime(year, month, dayOfWeek, dayOfMonth, hour, minute, second, msec);
	if(month<10){o="0";} else{o="";}
	if(dayOfWeek<10){d="0";} else{d="";}
	if(second<10){s="0";} else{s="";}
	if(minute<10){m="0";} else{m="";}
	
	
	setColor("white");
	setFont("SansSerif", 60, "bold");
	Overlay.drawString(title+"     "+year+"."+m+month+"."+d+dayOfMonth+" "+hour+":"+m+minute+":"+s+second+"     "+Otext, 10, hN);
	PX=newArray(1);
	PX[0]=w*0.88;
	PY=newArray(1);
	PY[0]=hN-80;
	makeSelection("point", PX, PY);
	run("Scale Bar...", "width=10 height=10 font=50 color=White background=None location=[At Selection] bold");
	Overlay.show();
	}

}
